#!/usr/bin/env python

# Copyright (C) 2002
# Max-Planck-Institut fuer Radioastronomie Bonn
#
# Produced for the ALMA and APEX projects
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Library General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Library General Public License for more
# details.
#
# You should have received a copy of the GNU Library General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 675 Massachusetts Ave, Cambridge, MA 02139, USA.  Correspondence concerning
# ALMA should be addressed as follows:
#
# Internet email: alma-sw-admin@nrao.edu
#
# Correspondence concerning APEX should be addressed as follows:
#
# Internet email: dmuders@mpifr-bonn.mpg.de

# The Python ALMA-TI FITS / APEX MBFITS Raw Data Writer
#
# Who			When		What
#
# C.Koenig, MPIfR       03/20/2003      Seperation of XmlSturucture from MBFits module
# C.Koenig, MPIfR	08/06/2003	- changed python interpreter to /usr/bin/env python
#					  (no longer python2.2) 
#					- changed last two lines of the module to comments:
#					  class XmlStructure shouldn't be used for object
#					  creation within this module
# C.Koenig, MPIfR       08/11/2003      XmlStructure now needs a defined FITS type as an
#                                       argument to work (no longer a file name)
# C.Koenig, MPIfR       09/02/2003      added attribute 'mode' to class XmlStructure

""" This is the ALMA-TI FITS / MBFITS XML Tools Library.
(c) 2002-2003 MPIfR """

import os
import config

class XmlStructure:
    def __init__(self, fitsType):

	if (fitsType == 'MBFITS'):
	    fileName = os.getenv('MBFITSXML')
            xmlFile = file(fileName)
	elif (fitsType == 'MAMBO'):
	    fileName = os.getenv('MAMBOXML')
            xmlFile = file(fileName)
	else:
            error = "Unknown FITS type."
	    raise error

        from xml.dom.minidom import parse, parseString

        # parse XML file by name
        dom = parse(xmlFile)

        # normalize whitespaces
        dom.documentElement.normalize()

        # first element is the Scan
        XmlScan=dom.documentElement
        self.mode = str(XmlScan.getAttribute('mode'))
        self.connectorModule = str(XmlScan.getAttribute('connectorModule'))
        
        # scan consists of table- and process-definitions
        if config.DEBUG:
            print "Reading XML: "+XmlScan.nodeName
        self.XmlScan=SubStructure(XmlScan)

class SubStructure:

    """ Analyze XML SubStructure """

    def __init__(self,process):
        self.create(process)
    
    def create(self,process):
        # analyze element (elements can have elements(nodeType 1),
	# attributes (nodeType 2), text (nodeType 3)
        if process.hasChildNodes():
            self.tables={}
            for element in process.childNodes:
                
                # create tables if one found in process definition
                if element.nodeType == 1 and element.getAttribute('content') == 'table':
                    self.tables[str(element.tagName)]=element
                    if config.DEBUG:
                        print "..... Table "+element.nodeName+" found in XML."

                # analyse substructure (process-definition) if one is found
                elif element.nodeType == 1 and element.getAttribute('content') == 'process':
                    struc=SubStructure(element)
                    for key in struc.tables.keys():
                        self.tables[key]=struc.tables[key]


# including the fits_writer module automaticaly creates the XmlStruc object instance
# XmlStruc=XmlStructure('MBFits.xml')
# xmlMambo=XmlStructure('Mambo.xml')
